#include "main.h"

extern int dt,
    size,
    num_explos,
    paused_game,
    current_level;
extern struct object objects[64];
extern struct explosion explosions[24];
extern struct object ship;
extern SDL_Surface *screen,
    *frame[NUM_BOOM_FRAMES],
    *obj[NUM_IMAGES];

extern Mix_Chunk *chunk[NUM_CHUNKS];

/* NAME:        rand_ab
 * DESCRIPTION: Losuje liczby z przedziau od 'a' do 'b'
 */
int rand_ab(int a, int b)
{
    return a + (int) ( ((double)(b-a+1))*rand()/(RAND_MAX+1.0));
}
/* NAME:	LoadImage
 * DESCRIPTION: laduje obrazek 	
 */
SDL_Surface *LoadImage(char *filename)
{
    SDL_Surface *tmp1,*tmp2;
    tmp1 = IMG_Load(filename);
    if(!tmp1)
	return NULL;
    tmp2 = SDL_DisplayFormat(tmp1);
    SDL_FreeSurface(tmp1);
    return tmp2;
}
/* NAME:	SetBGAlpha
 * DESCRIPTION:	Ustawia dla podanej powierzchni przezroczystosc 
 *		na kolorze z lewego gornego rogu
 */
void SetBGAlpha(SDL_Surface *img)
{
    int bpp = img->format->BytesPerPixel;
    Uint32 key = 0;

    switch(bpp) 
    {
	case 1:
	    key = ((Uint8*)img->pixels)[0];
	    break;

	case 2:
	    key = ((Uint16*)img->pixels)[0];
	    break;

	case 3:
		key |= ((Uint8*)img->pixels)[0];
		key <<= 8;
		key |= ((Uint8*)img->pixels)[1];
		key <<= 8;
		key |= ((Uint8*)img->pixels)[2];
	    break;

	case 4:
	    key = ((Uint32*)img->pixels)[0];
	    break;
    }
    SDL_SetColorKey(img, SDL_SRCCOLORKEY, key);

}
/* NAME:	MoveDown
 * DESCRIPTION:	Zmienia pozycje obiektu
 */
void MoveDown(struct object *obj)
{
    obj->rect.y=obj->rect.y+(int)(obj->velocity*dt);
}
/* NAME:        MoveUp
 * DESCRIPTION: Zmienia pozycje obiektu
 */
void MoveUp(struct object *obj)
{
    obj->rect.y=obj->rect.y-(int)(obj->velocity*dt);
}
/* NAME:	MoveRandomDown
 * DESCRIPTION:	Zmienia pozycje obiektu
 */
void MoveRandomDown(struct object *obj)
{
    obj->rect.y=obj->rect.y+(int)(obj->velocity*dt);

    switch(rand_ab(0,12))
    {
	case 0:
	    obj->direction = 0;
	    break;
	case 1:
	    obj->direction = 1;
	    break;
	case 2:
	    obj->direction = 2;
	    break;
	default:
	    break;
    }

    switch(obj->direction)
    {
	case 0:
	    obj->rect.x=obj->rect.x-(int)(obj->velocity*dt);
	    break;
	case 1:
	    obj->rect.x=obj->rect.x+(int)(obj->velocity*dt);
	    break;
	default:
	    break;
    }
}
/* NAME:        MoveRandomUp
 * DESCRIPTION: Zmienia pozycje obiektu
 */
void MoveRandomUp(struct object *obj)
{
    obj->rect.y=obj->rect.y-(int)(obj->velocity*dt);

    switch(rand_ab(0,80))
    {
	case 0:
	    obj->direction = 0;
	    break;
	case 1:
	    obj->direction = 1;
	    break;
	case 2:
	    obj->direction = 2;
	    break;
	default:
	    break;
    }

    switch(obj->direction)
    {
	case 0:
	    obj->rect.x=obj->rect.x-(int)(obj->velocity*dt);
	    break;
	case 1:
	    obj->rect.x=obj->rect.x+(int)(obj->velocity*dt);
	    break;
	default:
	    break;
    }
}
/* NAME:	MoveBOSS
 * DESCRIPTION:	
 */
void MoveBOSS(struct object *obj)
{
    if(obj->rect.y < obj->image->h + 10 && ship.rect.y > obj->rect.y) 
	obj->rect.y=obj->rect.y+(int)(obj->velocity*dt);
    else  
	switch(rand_ab(0,1))
	{
	    case 0:
		obj->rect.y=obj->rect.y-(int)(obj->velocity*dt);
		break;
	    default:
		if(ship.rect.y + ship.image->h < obj->rect.y)
		    obj->rect.y=obj->rect.y-(int)(obj->velocity*dt);
		else 
		    obj->rect.y=obj->rect.y+(int)(obj->velocity*dt);
		
		if(obj->rect.y + obj->image->h > ship.rect.y - 10)
		    obj->rect.y=obj->rect.y-(int)(obj->velocity*dt);
		break;
	}

    switch(rand_ab(0,1))
    {
	case 0:
	    obj->rect.x=obj->rect.x-(int)(obj->velocity*dt);
	    break;
	case 1:
	    obj->rect.x=obj->rect.x+(int)(obj->velocity*dt);
	    break;
	default:
	    break;
    }

    if(obj->rect.x + obj->image->w/2 > ship.rect.x + ship.image->w/2)
	obj->rect.x=obj->rect.x-(int)(obj->velocity*dt);

    if(obj->rect.x + obj->image->w/2 < ship.rect.x + ship.image->w/2)
	obj->rect.x=obj->rect.x+(int)(obj->velocity*dt);

}
/* NAME:	AddOBJ
 * DESCRIPTION:	Dodaje obiekt
 */
void AddOBJ(int x,int y,int type)
{
    objects[size].type = type;
    objects[size].remove = 0;

    switch(type) 
    {
	case BULLET:
	case MISSILE:
	    objects[size].rect.x = x;
	    objects[size].rect.y = y;	    

	    if(type==BULLET) {
		objects[size].direction = 2;
		objects[size].image = obj[0];
		objects[size].MoveOBJ = MoveDown;
	    } else {
		objects[size].direction = rand_ab(0,2); 
		objects[size].image = obj[12];
		objects[size].MoveOBJ = MoveRandomDown;
	    }

	    objects[size].velocity = 0.7; 
	    objects[size].energy = 1;
	    objects[size].shield = 0;
	    SetBGAlpha(objects[size].image);
	    break;
	case SHIP_BULLET:
	case SHIP_MISSILE:
	    if(type==SHIP_BULLET){
		objects[size].image = obj[0];
		objects[size].direction = 2;
		objects[size].MoveOBJ = MoveUp;
	    } else {
		objects[size].direction = 2;
		objects[size].image = obj[1];
		objects[size].MoveOBJ = MoveRandomUp;
	    }

	    objects[size].rect.x = ship.rect.x + ship.image->w/2 - 1;
	    objects[size].rect.y = ship.rect.y - objects[size].image->h;

	    SetBGAlpha(objects[size].image);
	    objects[size].energy=1;
	    objects[size].shield=0;
	    objects[size].velocity = 0.7;
	    break;
	case ENEMY:
	    switch(rand_ab(0,6))
	    {
		case 0:
		case 2:
		case 4:
		    objects[size].image = obj[3];
		    break;
		default:
		    objects[size].image = obj[2];
		    break;
	    }
	    SetBGAlpha(objects[size].image);

	    objects[size].rect.x = rand_ab(20,screen->w - objects[size].image->w - 20);
	    objects[size].rect.y = 0 - objects[size].image->h - 10;

	    objects[size].velocity = 0.1 * rand_ab(1,3);

	    objects[size].tols = SDL_GetTicks();

	    objects[size].energy = 100;
	    objects[size].shield = 0;

	    objects[size].direction = rand_ab(0,2);
	    objects[size].MoveOBJ = MoveRandomDown;  
	    break;
	case BOSS:
	    if(current_level%2 != 1)
		objects[size].image = obj[8];
	    else
		objects[size].image = obj[9];
	    SetBGAlpha(objects[size].image);

	    objects[size].rect.x = rand_ab(20,screen->w - objects[size].image->w - 20);
	    objects[size].rect.y = 0 - objects[size].image->h - 10;

	    objects[size].velocity = 0.2;

	    objects[size].tols = SDL_GetTicks();

	    objects[size].energy = 6000;
	    objects[size].shield = 0;

	    objects[size].MoveOBJ = MoveBOSS;

	    break;
	case PACK_LIVE:
	case PACK_ENERGY:
	case PACK_SHIELD:
	case PACK_SURPRISE:

	    objects[size].velocity = 0.2;
	    if(type==PACK_LIVE)
		objects[size].image = obj[4];
	    else if(type==PACK_SHIELD)
		objects[size].image = obj[5];
	    else if(type==PACK_ENERGY)
		objects[size].image = obj[6];
	    else if(type==PACK_SURPRISE)
		objects[size].image = obj[7];

	    objects[size].energy = 1;
	    objects[size].shield = 0;

	    objects[size].rect.x = rand_ab(20,600);
	    objects[size].rect.y = 0 - objects[size].image->h - 5;
	    objects[size].MoveOBJ = MoveDown;
	    break;
	default:
	    break;
    }
    size++;
}
/* NAME:	AddExplosion
 * DESCRIPTION:	Dodaje animowany wybuch
 */
void AddExplosion(int x,int y)
{
    explosions[num_explos].rect.x = x;
    explosions[num_explos].rect.y = y; 

    explosions[num_explos].starty = y;
    explosions[num_explos].startx = x;

    explosions[num_explos].frame = 0;
    explosions[num_explos].ttw = 35;
    explosions[num_explos].tolfc = SDL_GetTicks();

    num_explos++;
    Mix_PlayChannel(-1,chunk[3],0);
}
/* NAME:	LoadFrames
 * DESCRIPTION:	Laduje wszystkie ramki wybuchu
 */
void LoadFrames()
{
    int loop;
    char buf[100];
    char *name = "explos";

    for(loop=1;loop<10;loop++)
    {
	sprintf(buf,explosion_dir"%s0%d.png",name,loop);

	frame[loop-1] = LoadImage(buf);
	CheckSurface(frame[loop-1],"Unable to load %s: %s",buf ,SDL_GetError());
	SetBGAlpha(frame[loop-1]);
	SDL_SetAlpha(frame[loop-1],SDL_SRCALPHA,220);
    }
    for(loop=10;loop<=NUM_BOOM_FRAMES;loop++)
    {
	sprintf(buf,explosion_dir"%s%d.png",name,loop);
	frame[loop-1] = LoadImage(buf);
	CheckSurface(frame[loop-1],"Unable to load %s: %s",buf ,SDL_GetError());
	SetBGAlpha(frame[loop-1]);
	SDL_SetAlpha(frame[loop-1],SDL_SRCALPHA,220);
    }
}
/* NAME:	DeleteOBJ
 * DESCRIPTION:	Usuwa obiekt
 */
void DeleteOBJ(int which)
{
    memmove( objects+which, objects+which+1, (size-which-1)*EN_SIZE );
    size--;
}
/* NAME:        DeleteExplosion
 * DESCRIPTION: Usuwa wybuch
 */
void DeleteExplosion(int which)
{
    memmove( explosions+which, explosions+which+1, (num_explos-which-1)*EX_SIZE );
    num_explos--;
}
/* NAME:	DrawOBJ
 * DESCRIPTION:	rysuje obiekt
 */
void DrawOBJ(struct object obj)
{
    SDL_BlitSurface(obj.image,NULL,screen,&obj.rect);
}
/* NAME:	DrawExplosions
 * DESCRIPTION:	Rysuje wszystkie aktywne wybuchy
 */
void DrawExplosions()
{
    int i;
    for(i=0;i<num_explos;i++)
    {
	explosions[i].rect.x = explosions[i].startx - frame[explosions[i].frame]->w/2;
	explosions[i].rect.y = explosions[i].starty - frame[explosions[i].frame]->h/2;
	if(explosions[i].frame < 31)
	    SDL_BlitSurface(frame[explosions[i].frame],NULL,screen,&explosions[i].rect);

	if(!paused_game)
	if(SDL_GetTicks() - explosions[i].tolfc > explosions[i].ttw)
	{
	    explosions[i].frame++;
	    if(explosions[i].frame >= 31) {
		DeleteExplosion(i);
	    }
	    explosions[i].tolfc = SDL_GetTicks();
	}
    }
}
